@@ -139,8 +139,7 @@ static void exec_nonsimple_command(command_T *c, bool finally_exit)
 static void exec_simple_command(const commandinfo_T *ci,
 	int argc, char *argv0, void **argv, bool finally_exit)
     __attribute__((nonnull));
-static void print_xtrace(void *const *argv)
-    __attribute__((nonnull));
+static void print_xtrace(void *const *argv);
 static void exec_fall_back_on_sh(
 	int argc, char *const *argv, char *const *env, const char *path)
     __attribute__((nonnull(2,4)));
@@ -185,6 +184,10 @@ static bool exec_builtin_executed = false;
 /* the last assignment. */
 static assign_T *last_assign;
 
+/* a buffer for xtrace. */
+static xwcsbuf_T xtrace_buffer;
+
+
 /* Returns true iff we're breaking/continuing/returning now. */
 bool need_break(void)
 {
@@ -433,11 +436,16 @@ void exec_case(const command_T *c, bool finally_exit)
 	    size_t match = wfnmatch(pattern, word, 0, WFNM_WHOLE);
 	    free(pattern);
 	    if (match != WFNM_NOMATCH && match != WFNM_ERROR) {
-		exec_and_or_lists(ci->ci_commands, finally_exit);
-		goto done;
+		if (ci->ci_commands) {
+		    exec_and_or_lists(ci->ci_commands, finally_exit);
+		    goto done;
+		} else {
+		    goto success;
+		}
 	    }
 	}
     }
+success:
     laststatus = Exit_SUCCESS;
 done:
     if (finally_exit)
@@ -712,6 +720,7 @@ pid_t exec_process(command_T *c, exec_T type, pipeinfo_T *pi, pid_t pgid)
 		laststatus = lastcmdsubstatus;
 	    else
 		laststatus = Exit_ASSGNERR;
+	    print_xtrace(NULL);
 	} else {
 	    bool temp = c->c_assigns && assignment_is_temporary(cmdinfo.type);
 	    if (temp)
@@ -723,6 +732,7 @@ pid_t exec_process(command_T *c, exec_T type, pipeinfo_T *pi, pid_t pgid)
 				| (argc == 1 ? sct_argc1 : 0));
 		exec_simple_command(&cmdinfo, argc, argv0, argv, finally_exit);
 	    } else {
+		print_xtrace(NULL);
 		laststatus = Exit_ASSGNERR;
 		if (!is_interactive && cmdinfo.type == specialbuiltin)
 		    finally_exit = true;
@@ -1036,20 +1046,43 @@ void exec_simple_command(
 	exit_shell();
 }
 
+/* Returns a pointer to the xtrace buffer.
+ * The buffer is initialized if not. */
+xwcsbuf_T *get_xtrace_buffer(void)
+{
+    if (xtrace_buffer.contents == NULL)
+	wb_init(&xtrace_buffer);
+    return &xtrace_buffer;
+}
+
 /* Prints a trace if "-o xtrace" option is on. */
 void print_xtrace(void *const *argv)
 {
-    if (shopt_xtrace) {
+    bool tracevars = xtrace_buffer.contents != NULL
+		  && xtrace_buffer.length > 0;
+
+    if (shopt_xtrace && (tracevars || argv != NULL)) {
+	bool first = true;
+
 	print_prompt(4);
-	for (;;) {
-	    fprintf(stderr, "%ls", (wchar_t *) *argv);
-	    argv++;
-	    if (!*argv)
-		break;
-	    fputc(' ', stderr);
+	if (tracevars) {
+	    fprintf(stderr, "%ls", xtrace_buffer.contents + 1);
+	    first = false;
+	}
+	if (argv != NULL) {
+	    for (void *const *a = argv; *a != NULL; a++) {
+		if (!first)
+		    fputc(' ', stderr);
+		fprintf(stderr, "%ls", (wchar_t *) *a);
+		first = false;
+	    }
 	}
 	fputc('\n', stderr);
     }
+    if (xtrace_buffer.contents) {
+	wb_destroy(&xtrace_buffer);
+	xtrace_buffer.contents = NULL;
+    }
 }
 
 /* Executes a command script by `exec'ing a shell.
@@ -1197,7 +1230,7 @@ wchar_t *exec_command_substitution(const wchar_t *code)
 #if EAGAIN != EWOULDBLOCK
 		    case EWOULDBLOCK:
 #endif
-			wait_for_input(pipefd[PIDX_IN], false);
+			wait_for_input(pipefd[PIDX_IN], false, -1);
 			continue;
 		}
 		break;
@@ -1267,16 +1300,13 @@ int return_builtin(int argc, void **argv)
 
     int status;
     const wchar_t *statusstr = ARGV(xoptind);
-    if (statusstr == NULL) {
-	status = (savelaststatus >= 0) ? savelaststatus : laststatus;
+    if (statusstr != NULL
+	    && xwcstoi(statusstr, 10, &status)
+	    && status >= 0) {
+	status &= 0xFF;
     } else {
-	wchar_t *endofstr;
-	errno = 0;
-	status = (int) (wcstoul(statusstr, &endofstr, 0) & 0xFF);
-	if (errno || *endofstr != L'\0') {
-	    /* ignore `statusstr' if not a valid non-negative integer */
-	    status = (savelaststatus >= 0) ? savelaststatus : laststatus;
-	}
+	/* ignore `statusstr' if not a valid non-negative integer */
+	status = (savelaststatus >= 0) ? savelaststatus : laststatus;
     }
     execinfo.exception = ee_return;
     return status;
@@ -1315,17 +1345,19 @@ int break_builtin(int argc, void **argv)
     if (countstr == NULL) {
 	count = 1;
     } else {
-	wchar_t *endofstr;
-	errno = 0;
-	count = wcstoul(countstr, &endofstr, 0);
-	if (errno || *endofstr != L'\0') {
+	unsigned long countl;
+	if (!xwcstoul(countstr, 0, &countl)) {
 	    xerror(0, Ngt("`%ls' is not a valid integer"), countstr);
 	    SPECIAL_BI_ERROR;
 	    return Exit_ERROR;
-	} else if (count == 0) {
-	    xerror(0, Ngt("%u: not a positive integer"), count);
+	} else if (countl == 0) {
+	    xerror(0, Ngt("%u: not a positive integer"), 0u);
 	    SPECIAL_BI_ERROR;
 	    return Exit_ERROR;
+	} else if (countl > UINT_MAX) {
+	    count = UINT_MAX;
+	} else {
+	    count = (unsigned) countl;
 	}
     }
     assert(count > 0);
@@ -1946,4 +1978,4 @@ const char times_help[] = Ngt(
 );
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
