@@ -177,6 +177,8 @@ bool expand_line(void *const *restrict args,
     /* glob (list1 -> list2) */
     if (shopt_noglob) {
 	list2 = list1;
+	for (size_t i = 0; i < list2.length; i++)
+	    list2.contents[i] = unescapefree(list2.contents[i]);
     } else {
 	pl_init(&list2);
 	do_glob_each(pl_toary(&list1), &list2);
@@ -1856,4 +1858,4 @@ wchar_t *parse_and_expand_string(const wchar_t *s, const char *name, bool esc)
 }
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
