@@ -77,6 +77,9 @@ unsigned hist_next_number = 1;
 static unsigned max_number = MIN_MAX_NUMBER;
 /* The size limit of the history list. */
 static unsigned histsize = DEFAULT_HISTSIZE;
+/* The number of entries that are checked for duplicate when an entry is added.
+ */
+static unsigned histrmdup = 0;
 
 /* File stream for the history file. */
 static FILE *histfile = NULL;
@@ -147,6 +150,8 @@ static void write_histfile_pids(FILE *f)
 
 static void really_add_history(const wchar_t *line)
     __attribute__((nonnull));
+static void remove_dups(const char *line)
+    __attribute__((nonnull));
 
 
 /***** FORMAT OF THE HISTORY FILE *****
@@ -488,7 +493,6 @@ long read_signature(FILE *f)
     xwcsbuf_T buf;
     long rev = -1;
     const wchar_t *s;
-    wchar_t *end;
 
     wb_init(&buf);
     rewind(f);
@@ -499,9 +503,7 @@ long read_signature(FILE *f)
     if (!s || !iswdigit(s[0]))
 	goto end;
 
-    errno = 0;
-    rev = wcstol(s, &end, 10);
-    if (errno || *end != L'\0')
+    if (!xwcstol(s, 10, &rev))
 	rev = -1;
 end:
     wb_destroy(&buf);
@@ -637,7 +639,6 @@ void parse_process_id(const wchar_t *numstr)
     i = wcstoimax(numstr, &end, 10);
     if (errno || (*end != L'\0' && !iswspace(*end)))
 	return;
-
     if (i > 0)
 	add_histfile_pid((pid_t) i);
     else if (i < 0)
@@ -860,13 +861,18 @@ void maybe_init_history(void)
     const wchar_t *vhistsize = getvar(L VAR_HISTSIZE);
     if (vhistsize && vhistsize[0]) {
 	unsigned long size;
-	wchar_t *end;
-	errno = 0;
-	size = wcstoul(vhistsize, &end, 10);
-	if (!errno && !*end)
+	if (xwcstoul(vhistsize, 10, &size))
 	    set_histsize(size <= MAX_HISTSIZE ? size : MAX_HISTSIZE);
     }
 
+    /* set `histrmdup' */
+    const wchar_t *vhistrmdup = getvar(L VAR_HISTRMDUP);
+    if (vhistrmdup && vhistrmdup[0]) {
+	unsigned long rmdup;
+	if (xwcstoul(vhistrmdup, 10, &rmdup))
+	    histrmdup = rmdup <= histsize ? rmdup : histsize;
+    }
+
     update_time();
 
     /* open the history file and read it */
@@ -922,8 +928,8 @@ void finalize_history(void)
 
 /* Adds the specified `line' to the history.
  * Must not be called while the history is locked.
- * If `line' contains newlines, `line' is separated into multiple entries. */
-/* Only lines that contain graph-class characters are added to the history. */
+ * If `line' contains newlines, `line' is separated into multiple entries.
+ * Only lines that contain graph-class characters are added to the history. */
 void add_history(const wchar_t *line)
 {
     maybe_init_history();
@@ -955,11 +961,15 @@ void add_history(const wchar_t *line)
 /* Adds the specified `line' to the history.
  * `line' must not contain newlines.
  * `histfile' must be locked and `update_time' and `update_history' must have
- * been called. */
-/* If the `line' does not contain any graph-class characters, it is not added
- * to the history. */
+ * been called.
+ * If the `line' does not contain any graph-class characters, it is not added
+ * to the history. If the `shopt_histspace' option is enabled and the `line'
+ * starts with a space, the `line' is not added either. */
 void really_add_history(const wchar_t *line)
 {
+    if (shopt_histspace && line[0] == ' ')
+	return;
+
     /* Check if `line' contains `graph' characters */
     for (const wchar_t *s = line; ; ) {
 	if (*s == L'\0')
@@ -971,15 +981,37 @@ void really_add_history(const wchar_t *line)
 
     char *mbsline = malloc_wcstombs(line);
     if (mbsline) {
-	histentry_T *entry = new_entry(hist_next_number, now, mbsline);
+	histentry_T *entry;
 
+	remove_dups(mbsline);
+	entry = new_entry(hist_next_number, now, mbsline);
 	if (histfile)
 	    write_history_entry(entry, histfile);
-
 	free(mbsline);
     }
 }
 
+/* Removes entries whose value is the same as `line' in the `histrmdup' newest
+ * entries.
+ * `histfile' must be locked and `update_history' must have been called. The
+ * history file should be flushed after this function. */
+void remove_dups(const char *line)
+{
+    histentry_T *e = Histlist;
+    histentry_T *saveprev = e->Prev;
+    for (unsigned i = histrmdup; i > 0; i--) {
+	e = saveprev;
+	saveprev = e->Prev;
+	if (e == Histlist)
+	    break;
+	if (strcmp(e->value, line) == 0) {
+	    if (histfile)
+		fwprintf(histfile, L"d%X\n", e->number);
+	    remove_entry(e);
+	}
+    }
+}
+
 /* Returns the history entry that has the specified `number', or NULL if there
  * is no such entry. */
 const histentry_T *get_history_entry(unsigned number)
@@ -1140,10 +1172,7 @@ int fc_builtin(int argc, void **argv)
     if (silent && vlast)
 	goto print_usage;
     if (vfirst) {
-	wchar_t *end;
-	errno = 0;
-	first = wcstol(vfirst, &end, 10);
-	if (!vfirst[0] || errno || *end || first == 0) {
+	if (!xwcstol(vfirst, 10, &first) || first == 0) {
 	    efirst = fc_search_entry(vfirst);
 	    if (efirst == Histlist)
 		return Exit_FAILURE;
@@ -1157,10 +1186,7 @@ int fc_builtin(int argc, void **argv)
 	first = list ? -16 : -1;
     }
     if (vlast) {
-	wchar_t *end;
-	errno = 0;
-	last = wcstol(vlast, &end, 10);
-	if (!vlast[0] || errno || *end || last == 0) {
+	if (!xwcstol(vlast, 10, &last) || last == 0) {
 	    elast = fc_search_entry(vlast);
 	    if (elast == Histlist)
 		return Exit_FAILURE;
@@ -1485,4 +1511,4 @@ const char fc_help[] = Ngt(
 );
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
