@@ -185,7 +185,7 @@ start:
 	    case EWOULDBLOCK:
 #endif
 		clearerr(f);
-		wait_for_input(fd, true);
+		wait_for_input(fd, true, -1);
 		goto start;
 	    default:
 		xerror(errno, Ngt("cannot read input"));
@@ -238,7 +238,7 @@ bool read_line_from_stdin(struct xwcsbuf_T *buf, bool trap)
 #if EAGAIN != EWOULDBLOCK
 	    case EWOULDBLOCK:
 #endif
-		wait_for_input(STDIN_FILENO, trap);
+		wait_for_input(STDIN_FILENO, trap, -1);
 		break;
 	    default:
 		xerror(errno, Ngt("cannot read input"));
@@ -308,10 +308,8 @@ int input_readline(struct xwcsbuf_T *buf, void *inputinfo)
 		if (info->type == 1)
 		    info->type = 2;
 		if (line[0]) {
-		    size_t len = wcslen(line);
-		    assert(len > 0);
 #if YASH_ENABLE_HISTORY
-		    if (info->type == 2 && len > 1)
+		    if (info->type == 2)
 			add_history(line);
 #endif /* YASH_ENABLE_HISTORY */
 		    linebuffer = forward_line(line, buf);
@@ -343,8 +341,7 @@ int input_readline(struct xwcsbuf_T *buf, void *inputinfo)
 	result = input_file(buf, info->fp);
 #if YASH_ENABLE_HISTORY
     if (info->type == 2)
-	if (buf->contents[oldlen] != L'\n' && buf->length > oldlen)
-	    add_history(buf->contents + oldlen);
+	add_history(buf->contents + oldlen);
 #endif
     return result;
 }
@@ -609,4 +606,4 @@ bool unset_nonblocking(int fd)
 }
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
