@@ -25,6 +25,7 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/wait.h>
 #include <unistd.h>
 #include <wctype.h>
@@ -679,16 +680,22 @@ void notify_signaled_job(size_t jobnumber)
 	return;
 
     int sig = WTERMSIG(p->pr_statuscode);
-    if (sig == SIGINT || sig == SIGPIPE)
-	return;
-
+    switch (sig) {
+	case SIGINT:
+	    fputc('\n', stderr);
+	    break;
+	case SIGPIPE:
+	    break;
+	default:
 #if HAVE_STRSIGNAL
-    fprintf(stderr, gt("Process killed by SIG%s: %s\n"),
-	    get_signal_name(sig), strsignal(sig));
+	    fprintf(stderr, gt("Process killed by SIG%s: %s\n"),
+		    get_signal_name(sig), strsignal(sig));
 #else
-    fprintf(stderr, gt("Process killed by SIG%s\n"),
-	    get_signal_name(sig));
+	    fprintf(stderr, gt("Process killed by SIG%s\n"),
+		    get_signal_name(sig));
 #endif
+	    break;
+    }
 }
 
 /* Returns the job number from the specified job ID string.
@@ -709,12 +716,9 @@ size_t get_jobnumber_from_name(const wchar_t *name)
 	return previous_jobnumber;
 
     if (iswdigit(name[0])) {
-	unsigned num;
-	wchar_t *nameend;
-	errno = 0;
-	num = wcstoul(name, &nameend, 10);
-	if (!errno && !*nameend)
-	    return get_job(num) ? num : 0;
+	unsigned long num;
+	if (xwcstoul(name, 10, &num))
+	    return (num <= SIZE_MAX && get_job(num)) ? num : 0;
     }
 
     bool contain;
@@ -1039,10 +1043,7 @@ int wait_builtin(int argc, void **argv)
 		jobnumber = get_jobnumber_from_name(jobspec + 1);
 	    } else {
 		long pid;
-		wchar_t *end;
-		errno = 0;
-		pid = wcstol(jobspec, &end, 10);
-		if (errno || *end || pid < 0) {
+		if (!xwcstol(jobspec, 10, &pid) || pid < 0) {
 		    xerror(0, Ngt("%ls: invalid job specification"), jobspec);
 		    err = true;
 		    continue;
@@ -1184,4 +1185,4 @@ const char disown_help[] = Ngt(
 );
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
