@@ -1089,7 +1089,7 @@ void cmd_redraw_all(wchar_t c __attribute__((unused)))
 {
     le_display_clear();
     le_restore_terminal();
-    le_setupterm();
+    le_setupterm(false);
     le_set_terminal();
     le_display_print_all(false);
 }
@@ -1970,8 +1970,8 @@ void cmd_vi_exec_alias(wchar_t c __attribute__((unused)))
 }
 
 /* Appends the value of the alias `_c' to the pre-buffer so that the alias value
- * is interpreted as commands, where `c' in the alias name is the argument of this
- * command. */
+ * is interpreted as commands, where `c' in the alias name is the argument of
+ * this command. */
 void vi_exec_alias(wchar_t c)
 {
     le_set_mode(LE_MODE_VI_COMMAND);
@@ -2064,7 +2064,7 @@ end:
 	free(tempfile);
 	le_resume_readline();
     } else {  // child process
-	fwprintf(f, L"%ls\n");
+	fwprintf(f, L"%ls\n", le_main_buffer.contents);
 	fclose(f);
 
 	wchar_t *command = malloc_wprintf(L"vi %s", tempfile);
@@ -2470,4 +2470,4 @@ done:
 }
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
