@@ -47,6 +47,7 @@ extern void le_resume_readline(void);
 extern void le_display_size_changed(void);
 
 extern _Bool le_next_verbatim;
+extern int le_read_timeout;
 
 void append_to_prebuffer(char *s)
     __attribute__((nonnull));
@@ -55,4 +56,4 @@ void append_to_prebuffer(char *s)
 #endif /* YASH_LINEEDIT_H */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
