@@ -122,15 +122,11 @@ const char *get_signal_name(int signum)
 int get_signal_number(const char *name)
 {
     if (isdigit(name[0])) {  /* parse a decimal integer */
-	long num;
-	char *end;
-	errno = 0;
-	num = strtol(name, &end, 10);
-	if (errno || *end != '\0' || num < 0 || num > INT_MAX)
+	int signum;
+	if (!xstrtoi(name, 10, &signum) || signum < 0)
 	    return -1;
 
-	/* check if `num' is a valid signal */
-	int signum = (int) num;
+	/* check if `signum' is a valid signal */
 	if (signum == 0)
 	    return 0;
 #if defined SIGRTMIN && defined SIGRTMAX
@@ -157,13 +153,11 @@ int get_signal_number(const char *name)
 	if (name[0] == '\0') {
 	    return sigrtmin;
 	} else if (name[0] == '+') {
-	    long num;
-	    char *end;
-	    errno = 0;
-	    num = strtol(name, &end, 10);
-	    if (errno || *end != '\0' || num < 0 || sigrtmin + num >= SIGRTMAX)
+	    int num;
+	    if (!xstrtoi(name, 10, &num)
+		    || num < 0 || SIGRTMAX - sigrtmin < num)
 		return -1;
-	    return sigrtmin + (int) num;
+	    return sigrtmin + num;
 	}
     } else if (strncmp(name, "RTMAX", 5) == 0) {
 	int sigrtmax = SIGRTMAX;
@@ -171,13 +165,11 @@ int get_signal_number(const char *name)
 	if (name[0] == '\0') {
 	    return sigrtmax;
 	} else if (name[0] == '-') {
-	    long num;
-	    char *end;
-	    errno = 0;
-	    num = strtol(name, &end, 10);
-	    if (errno || *end != '\0' || num > 0 || sigrtmax + num < SIGRTMIN)
+	    int num;
+	    if (!xstrtoi(name, 10, &num)
+		    || num > 0 || SIGRTMIN - sigrtmax > num)
 		return -1;
-	    return sigrtmax + (int) num;
+	    return sigrtmax + num;
 	}
     }
 #endif
@@ -563,10 +555,28 @@ finish:
 /* Waits for the specified file descriptor to be available for reading.
  * `handle_sigchld' and `handle_sigwinch' are called to handle SIGCHLD and
  * SIGWINCH.
- * If `trap' is true, traps are also handled while waiting. */
-void wait_for_input(int fd, bool trap)
+ * If `trap' is true, traps are also handled while waiting.
+ * The maximum time length of wait is specified by `timeout' in milliseconds.
+ * If `timeout' is negative, the time length is unlimited.
+ * If the wait is interrupted by a signal, this function will re-wait for the
+ * specified timeout, which means that this function may wait for a time length
+ * longer than the specified timeout.
+ * This function returns true if the input is ready, and false if an error
+ * occurred or it timed out. */
+bool wait_for_input(int fd, bool trap, int timeout)
 {
+    bool success;
     sigset_t ss, savess;
+    struct timespec to;
+    struct timespec *top;
+
+    if (timeout < 0) {
+	top = NULL;
+    } else {
+	to.tv_sec  = timeout / 1000;
+	to.tv_nsec = timeout % 1000 * 1000000;
+	top = &to;
+    }
 
 start:
     sigfillset(&ss);
@@ -589,12 +599,13 @@ start:
 	fd_set fdset;
 	FD_ZERO(&fdset);
 	FD_SET(fd, &fdset);
-	if (pselect(fd + 1, &fdset, NULL, NULL, NULL, &savess) >= 0) {
-	    if (FD_ISSET(fd, &fdset))
-		break;
+	if (pselect(fd + 1, &fdset, NULL, NULL, top, &savess) >= 0) {
+	    success = FD_ISSET(fd, &fdset);
+	    break;
 	} else {
 	    if (errno != EINTR) {
 		xerror(errno, "pselect");
+		success = false;
 		break;
 	    }
 	}
@@ -602,6 +613,8 @@ start:
 
     if (sigprocmask(SIG_SETMASK, &savess, NULL) < 0)
 	xerror(errno, "sigprocmask");
+
+    return success;
 }
 
 /* Handles SIGCHLD if caught. */
@@ -1173,23 +1186,20 @@ main:
 #endif
 	} else {
 	    do {
-		long signum;
-		wchar_t *end;
+		int signum;
 
-		errno = 0;
-		signum = wcstol(ARGV(xoptind), &end, 10);
-		if (errno || *end || signum < 0 || signum > INT_MAX) {
-		    xerror(0, Ngt("`%ls' is not a valid integer"),
-			    ARGV(xoptind));
-		    err = true;
-		    continue;
+		if (xwcstoi(ARGV(xoptind), 10, &signum) && signum >= 0) {
+		    if (signum >= TERMSIGOFFSET)
+			signum -= TERMSIGOFFSET;
+		    else if (signum >= (TERMSIGOFFSET & 0xFF))
+			signum -= (TERMSIGOFFSET & 0xFF);
+		} else {
+		    signum = get_signal_number_w(ARGV(xoptind));
 		}
-		if (signum >= TERMSIGOFFSET)
-		    signum -= TERMSIGOFFSET;
-		else if (signum >= (TERMSIGOFFSET & 0xFF))
-		    signum -= (TERMSIGOFFSET & 0xFF);
-		if (signum <= 0 || !print_signal((int) signum, NULL, verbose))
+		if (signum <= 0 || !print_signal(signum, NULL, verbose)) {
 		    xerror(0, Ngt("%ls: no such signal"), ARGV(xoptind));
+		    err = true;
+		}
 	    } while (++xoptind < argc);
 	}
     } else {
@@ -1203,11 +1213,8 @@ main:
 		    err = true;
 	    } else {
 		long pid;
-		wchar_t *end;
 
-		errno = 0;
-		pid = wcstol(proc, &end, 10);
-		if (errno || *end) {
+		if (!xwcstol(proc, 10, &pid)) {
 		    xerror(0, Ngt("`%ls' is not a valid integer"), proc);
 		    err = true;
 		    continue;
@@ -1304,4 +1311,4 @@ const char kill_help[] = Ngt(
 );
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
