@@ -19,6 +19,7 @@
 #ifndef YASH_UTIL_H
 #define YASH_UTIL_H
 
+#include <stdint.h>
 #include <stdlib.h>
 
 #define Size_max ((size_t) -1)  // = SIZE_MAX
@@ -77,6 +78,14 @@ extern wchar_t *xwcsndup(const wchar_t *s, size_t maxlen)
     __attribute__((malloc,warn_unused_result,nonnull));
 static inline wchar_t *xwcsdup(const wchar_t *s)
     __attribute__((malloc,warn_unused_result,nonnull));
+extern _Bool xstrtoi(const char *s, int base, int *resultp)
+    __attribute__((warn_unused_result,nonnull));
+extern _Bool xwcstoi(const wchar_t *s, int base, int *resultp)
+    __attribute__((warn_unused_result,nonnull));
+extern _Bool xwcstol(const wchar_t *s, int base, long *resultp)
+    __attribute__((warn_unused_result,nonnull));
+extern _Bool xwcstoul(const wchar_t *s, int base, unsigned long *resultp)
+    __attribute__((warn_unused_result,nonnull));
 static inline void **duparray(void *const *array, void *copy(const void *p))
     __attribute__((malloc,warn_unused_result,nonnull(2)));
 extern void **duparrayn(
@@ -98,12 +107,18 @@ extern void sort_mbs_array(void **array)
     __attribute__((nonnull));
 
 #if HAVE_STRNLEN
+# if _POSIX_C_SOURCE < 200809L
+# undef strnlen
 extern size_t strnlen(const char *s, size_t maxlen);
-#define xstrnlen(s,maxlen) strnlen(s,maxlen)
+# endif
+# define xstrnlen(s,maxlen) strnlen(s,maxlen)
 #endif
 #if HAVE_WCSNLEN
+# if _POSIX_C_SOURCE < 200809L
+# undef wcsnlen
 extern size_t wcsnlen(const wchar_t *s, size_t maxlen);
-#define xwcsnlen(s,maxlen) wcsnlen(s,maxlen)
+# endif
+# define xwcsnlen(s,maxlen) wcsnlen(s,maxlen)
 #endif
 
 /* Returns a newly malloced copy of the specified string.
@@ -212,4 +227,4 @@ wchar_t xgetopt(void **restrict argv, const wchar_t *restrict optstring)
 #endif /* YASH_UTIL_H */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
