@@ -20,6 +20,7 @@
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
+#include <inttypes.h>
 #include <limits.h>
 #include <locale.h>
 #include <stdbool.h>
@@ -47,6 +48,10 @@
 #include "variable.h"
 #include "version.h"
 #include "yash.h"
+#if YASH_ENABLE_LINEEDIT
+# include "lineedit/lineedit.h"
+# include "lineedit/terminfo.h"
+#endif
 
 
 const wchar_t *const path_variables[PA_count] = {
@@ -141,9 +146,9 @@ static variable_T *new_temporary(const wchar_t *name)
     __attribute__((nonnull));
 static variable_T *new_variable(const wchar_t *name, scope_T scope)
     __attribute__((nonnull));
-static void print_variable_trace(const wchar_t *name, const wchar_t *value)
+static void xtrace_variable(const wchar_t *name, const wchar_t *value)
     __attribute__((nonnull));
-static void print_array_trace(const wchar_t *name, void *const *values)
+static void xtrace_array(const wchar_t *name, void *const *values)
     __attribute__((nonnull));
 static void get_all_variables_rec(hashtable_T *table, environ_T *env)
     __attribute__((nonnull));
@@ -319,6 +324,13 @@ void init_variables(void)
 	random_active = false;
     }
 
+    /* set $YASH_LE_TIMEOUT */
+#if YASH_ENABLE_LINEEDIT
+    if (is_interactive)
+	set_variable(L VAR_YASH_LE_TIMEOUT,
+		xwcsdup(L"100"), SCOPE_GLOBAL, false);
+#endif
+
     /* set $YASH_VERSION */
     set_variable(L VAR_YASH_VERSION, xwcsdup(L PACKAGE_VERSION),
 	    SCOPE_GLOBAL, false);
@@ -705,7 +717,7 @@ bool do_assignments(const assign_T *assign, bool temp, bool export)
 		    return false;
 		value = unescapefree(value);
 		if (shopt_xtrace)
-		    print_variable_trace(assign->a_name, value);
+		    xtrace_variable(assign->a_name, value);
 		if (!set_variable(assign->a_name, value, scope, export))
 		    return false;
 		break;
@@ -714,7 +726,7 @@ bool do_assignments(const assign_T *assign, bool temp, bool export)
 		    return false;
 		assert(values != NULL);
 		if (shopt_xtrace)
-		    print_array_trace(assign->a_name, values);
+		    xtrace_array(assign->a_name, values);
 		if (!set_array(assign->a_name, count, values, scope))
 		    return false;
 		break;
@@ -724,28 +736,29 @@ bool do_assignments(const assign_T *assign, bool temp, bool export)
     return true;
 }
 
-/* Prints trace of an variable assignment. */
-void print_variable_trace(const wchar_t *name, const wchar_t *value)
+/* Pushes a trace of an variable assignment to the xtrace buffer. */
+void xtrace_variable(const wchar_t *name, const wchar_t *value)
 {
-    print_prompt(4);
-    fprintf(stderr, "%ls=%ls\n", name, value);
+    xwcsbuf_T *buf = get_xtrace_buffer();
+    wb_wprintf(buf, L" %ls=%ls", name, value);
 }
 
-/* Prints trace of an array assignment. */
-void print_array_trace(const wchar_t *name, void *const *values)
+/* Pushes a trace of an array assignment to the xtrace buffer. */
+void xtrace_array(const wchar_t *name, void *const *values)
 {
-    print_prompt(4);
-    fprintf(stderr, "%ls=(", name);
+    xwcsbuf_T *buf = get_xtrace_buffer();
+
+    wb_wprintf(buf, L" %ls=(", name);
     if (*values) {
 	for (;;) {
-	    fprintf(stderr, "%ls", (wchar_t *) *values);
+	    wb_cat(buf, *values);
 	    values++;
 	    if (!*values)
 		break;
-	    fputc(' ', stderr);
+	    wb_wccat(buf, L' ');
 	}
     }
-    fputs(")\n", stderr);
+    fputs(")", stderr);
 }
 
 /* Gets the value of the specified scalar variable.
@@ -831,12 +844,12 @@ positional_parameters:
 	/* `name' starts with a digit: a positional parameter */
 	wchar_t *nameend;
 	errno = 0;
-	long v = wcstol(name, &nameend, 10);
+	uintmax_t v = wcstoumax(name, &nameend, 10);
 	if (errno || *nameend != L'\0')
 	    return result;  /* not a number or overflow */
 	var = search_variable(L VAR_positional);
 	assert(var != NULL && (var->v_type & VF_MASK) == VF_ARRAY);
-	if (v <= 0 || (uintmax_t) var->v_valc < (uintmax_t) v)
+	if (v == 0 || var->v_valc < v)
 	    return result;  /* index out of bounds */
 	value = xwcsdup(var->v_vals[v - 1]);
 	goto return_single;
@@ -1007,17 +1020,30 @@ void variable_set(const wchar_t *name, variable_T *var)
 	if (random_active && wcscmp(name, L VAR_RANDOM) == 0) {
 	    random_active = false;
 	    if (var && (var->v_type & VF_MASK) == VF_NORMAL && var->v_value) {
-		wchar_t *end;
-		errno = 0;
-		unsigned seed = wcstoul(var->v_value, &end, 0);
-		if (!errno && *end == L'\0') {
-		    srand(seed);
+		unsigned long seed;
+		if (xwcstoul(var->v_value, 0, &seed)) {
+		    srand((unsigned) seed);
 		    var->v_getter = random_getter;
 		    random_active = true;
 		}
 	    }
 	}
 	break;
+#if YASH_ENABLE_LINEEDIT
+    case L'T':
+	if (wcscmp(name, L VAR_TERM) == 0)
+	    le_need_term_update = true;
+	break;
+    case L'Y':
+	if (wcscmp(name, L VAR_YASH_LE_TIMEOUT) == 0) {
+	    if (var && (var->v_type & VF_MASK) == VF_NORMAL && var->v_value) {
+		int v;
+		if (xwcstoi(var->v_value, 0, &v))
+		    le_read_timeout = v;
+	    }
+	}
+	break;
+#endif /* YASH_ENABLE_LINEEDIT */
     }
 }
 
@@ -1293,13 +1319,10 @@ bool parse_dirstack_index(const wchar_t *indexstr,
 	size_t *indexp, const wchar_t **entryp, bool printerror)
 {
     long num;
-    wchar_t *end;
 
     if (indexstr[0] != L'-' && indexstr[0] != L'+')
 	goto not_index;
-    errno = 0;
-    num = wcstol(indexstr, &end, 10);
-    if (errno || *end)
+    if (!xwcstol(indexstr, 10, &num))
 	goto not_index;
 
     variable_T *var = search_variable(L VAR_DIRSTACK);
@@ -1912,10 +1935,7 @@ bool array_remove_elements(
     /* convert all the strings into long integers */
     for (size_t i = 0; i < count; i++) {
 	const wchar_t *indexwcs = indexwcss[i];
-	wchar_t *end;
-	errno = 0;
-	indices[i] = wcstol(indexwcs, &end, 10);
-	if (errno || !indexwcs[0] || end[0]) {
+	if (!xwcstol(indexwcs, 10, &indices[i])) {
 	    xerror(errno, Ngt("`%ls' is not a valid integer"), indexwcs);
 	    return false;
 	}
@@ -1976,10 +1996,7 @@ bool array_insert_elements(
     assert(values[0] != NULL);
     {
 	const wchar_t *indexword = *values;
-	wchar_t *end;
-	errno = 0;
-	index = wcstol(indexword, &end, 10);
-	if (errno || !indexword[0] || end[0]) {
+	if (!xwcstol(indexword, 10, &index)) {
 	    xerror(errno, Ngt("`%ls' is not a valid integer"), indexword);
 	    return false;
 	}
@@ -2010,17 +2027,12 @@ bool array_insert_elements(
 bool array_set_element(const wchar_t *name, variable_T *array,
 	const wchar_t *indexword, const wchar_t *value)
 {
-    long index;
-
     assert((array->v_type & VF_MASK) == VF_ARRAY);
-    {
-	wchar_t *end;
-	errno = 0;
-	index = wcstol(indexword, &end, 10);
-	if (errno || !indexword[0] || end[0]) {
-	    xerror(errno, Ngt("`%ls' is not a valid integer"), indexword);
-	    return false;
-	}
+
+    long index;
+    if (!xwcstol(indexword, 10, &index)) {
+	xerror(errno, Ngt("`%ls' is not a valid integer"), indexword);
+	return false;
     }
 
     size_t uindex;
@@ -2191,10 +2203,7 @@ int shift_builtin(int argc, void **argv)
     size_t scount;
     if (xoptind < argc) {
 	long count;
-	wchar_t *end;
-	errno = 0;
-	count = wcstol(ARGV(xoptind), &end, 10);
-	if (*end || errno) {
+	if (!xwcstol(ARGV(xoptind), 10, &count)) {
 	    xerror(errno, Ngt("`%ls' is not a valid integer"), ARGV(xoptind));
 	    SPECIAL_BI_ERROR;
 	    return Exit_ERROR;
@@ -2267,13 +2276,9 @@ int getopts_builtin(int argc, void **argv)
     }
     {
 	const wchar_t *varoptind = getvar(L VAR_OPTIND);
-	wchar_t *end;
-	if (!varoptind || !varoptind[0])
-	    goto optind_invalid;
-	errno = 0;
-	optind = wcstoul(varoptind, &end, 10) - 1;
-	if (errno || *end)
+	if (!varoptind || !xwcstoul(varoptind, 10, &optind))
 	    goto optind_invalid;
+	optind -= 1;
     }
     if (xoptind < argc) {
 	if (optind >= (unsigned long) (argc - xoptind))
@@ -2717,4 +2722,4 @@ const char dirs_help[] = Ngt(
 #endif /* YASH_ENABLE_DIRSTACK */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
