@@ -31,6 +31,7 @@ extern char **environ;
 #define VAR_ENV		"ENV"
 #define VAR_FCEDIT	"FCEDIT"
 #define VAR_HISTFILE	"HISTFILE"
+#define VAR_HISTRMDUP   "HISTRMDUP"
 #define VAR_HISTSIZE	"HISTSIZE"
 #define VAR_HOME	"HOME"
 #define VAR_IFS		"IFS"
@@ -61,6 +62,7 @@ extern char **environ;
 #define VAR_PWD		"PWD"
 #define VAR_RANDOM	"RANDOM"
 #define VAR_TERM	"TERM"
+#define VAR_YASH_LE_TIMEOUT "YASH_LE_TIMEOUT"
 #define VAR_YASH_VERSION "YASH_VERSION"
 #define L               L""
 
@@ -144,4 +146,4 @@ extern const char typeset_help[], array_help[], unset_help[], shift_help[],
 #endif /* YASH_VARIABLE_H */
 
 
-/* vim: set ts=8 sts=4 sw=4 noet: */
+/* vim: set ts=8 sts=4 sw=4 noet tw=80: */
